<?php

/**
 * The admin-specific functionality of Visidea.
 *
 * @link       https://visidea.ai
 * @since      1.0.0
 *
 * @package    Visidea
 * @subpackage Visidea/admin
 */

class Visidea_Admin {

  /**
   * The ID of Visidea.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $plugin_name    The ID of this plugin.
   */
  private $plugin_name;

  /**
   * The version of this plugin.
   *
   * @since    1.0.0
   * @access   private
   * @var      string    $version    The current version of this plugin.
   */
  private $version;

  /**
   * Initialize the class and set its properties.
   *
   * @since    1.0.0
   * @param      string    $plugin_name       The name of this plugin.
   * @param      string    $version    The version of this plugin.
   */
  public function __construct($plugin_name, $version) {
  	$this->plugin_name = $plugin_name;
  	$this->version = $version;
  }

  public function cron_items_dump() {
    // error_log( 'fired cron_items_dump at ' . date( DATE_RFC2822 ) );

    // Re-initialize items dump file
    $options = get_option('visidea_plugin_options');
    $option_filename = 'items_'.$options['private_token'].'.csv';
    $file = plugin_dir_path(__FILE__) . $option_filename;

    $option_default_columns = get_option('ry_items_columns');
    $fp = fopen($file, 'w');
    // Write default columns
    fwrite($fp, $option_default_columns);

    $args = array(
      'limit' => -1,
      'status' => 'publish',
      // 'return' => 'Objects',
    );

    $products = wc_get_products($args);
    //var_dump($products);

    foreach ($products as $item){

      $buffer = $item->get_id() . ';' .
        Visidea::sanitizeString($item->get_name()) . ';' .
        '"";'.'"";';
      if ($item->is_type('variable')) {
        $discount = (($item->get_variation_regular_price('min') - $item->get_variation_price('min'))/ $item->get_variation_regular_price('min')) * 100;

        if (is_nan($discount) || $discount == 100)
            $discount = 0;

        $buffer = $buffer . $item->get_variation_regular_price('min'). ';' .
                    $item->get_variation_price('min'). ';' .
                    $discount . ';';

      } else if ($item->is_type('grouped')) {
        $min_price = INF;
        $min_discount_price = INF;

        foreach ($item->get_children() as $child_id ){
          $child = wc_get_product( $child_id);
          if ($child->get_price() < $min_discount_price){
            $min_discount_price = $child->get_price();
            $min_price = $child->get_regular_price();
          }
        }

        $discount = (($min_price - $min_discount_price) / $min_price) * 100;

        if (is_nan($discount) || $discount == 100)
          $discount = 0;

        $buffer = $buffer . $min_price . ';' .
        $min_discount_price . ';' .
        $discount . ';';

      } else {
        $discount = (($item->get_regular_price() - $item->get_price())/ $item->get_regular_price()) * 100;

        if (is_nan($discount) || $discount == 100)
          $discount = 0;

        $buffer = $buffer . $item->get_regular_price() . ';' .
        $item->get_price(). ';' .
        $discount . ';';
      }

      // Create Category ids + names fields
      $category_ids = "";
      $category_names = "";
      $items = count($item->get_category_ids());
      $i = 0;
      foreach ($item->get_category_ids() as $category_id) {
        $last_iteration = !(--$i);
        $category_ids = $category_ids . $category_id;

        // Get Category name
        $category_name = get_term_by('id', $category_id, 'product_cat' )->name;
        $category_names = $category_names . $category_name;
        if (!$last_iteration) {
          $category_ids = $category_ids . "|";
          $category_names = $category_names . "|";
        }
      }

      $buffer = $buffer. $category_ids . ';' .
        Visidea::sanitizeString($category_names) . ';' .
        Visidea::sanitizeString(get_permalink($item->get_id())). ';' .
        Visidea::sanitizeString(wp_get_attachment_url($item->get_image_id())) . "\n";
      fwrite($fp, $buffer);
    }

    fclose($fp);

  }

  public function cron_users_dump() {
    // error_log( 'fired cron_users_dump at ' . date( DATE_RFC2822 ) );

    // Re-initialize items dump file
    $options = get_option('visidea_plugin_options');
    $option_filename = 'users_'.$options['private_token'].'.csv';
    $file = plugin_dir_path(__FILE__) . $option_filename;

    $option_default_columns = get_option('ry_users_columns');
    $fp = fopen($file, 'w');
    // Write default columns
    fwrite($fp, $option_default_columns);

    // Obtain customers dump
    $args = array(
      'fields' => 'all_with_meta',
      'role' => 'Customer',
    );
    $customers = get_users($args);

    // Write dump
    foreach ($customers as $user) {
        $buffer = $user->ID.";".
        Visidea::sanitizeString($user->user_email).';'.
        Visidea::sanitizeString($user->first_name).';'.
        Visidea::sanitizeString($user->last_name).';'.
        Visidea::sanitizeString($user->billing_address_1).';'.
        Visidea::sanitizeString($user->billing_city).';'.
        Visidea::sanitizeString($user->billing_postcode).';'.
        Visidea::sanitizeString($user->billing_state).';'.
        Visidea::sanitizeString($user->billing_country).';'.
        Visidea::sanitizeString(get_user_locale($user->ID)).';'.
        Visidea::sanitizeString($user->billing_country).';'.
        wc_get_customer_total_spent($user->ID).';'.
        wc_get_customer_order_count($user->ID).';'.
        Visidea::sanitizeString($user->user_registered) . "\n";
        fwrite($fp, $buffer);
    }
    fclose($fp);

  }

  public function cron_interactions_dump() {
      // error_log( 'fired cron_interactions_dump at ' . date( DATE_RFC2822 ) );

      // Initialize interactions file
      $options = get_option('visidea_plugin_options');
      $option_filename = 'interactions_'.$options['private_token'].'.csv';
      $file = plugin_dir_path(__FILE__) . $option_filename;
      $fp = fopen($file, 'w');

      $option_default_columns = get_option('ry_interactions_columns');
      fwrite($fp, $option_default_columns);

      // Get all previous orders
      $query = new WC_Order_Query( array(
        'limit' => -1,
        'orderby' => 'date',
        'order' => 'DESC',
      ));

      $orders = $query->get_orders();

      foreach ($orders as $order) {

        // Get user ID
        $user_id = $order->get_user_id();
        $customer = new WC_Customer($user_id);
        $role = $customer->get_role();

        // If the user is not a customer (its an admin) return
        if ($role == "customer") {

          // Loop through order items and write them in the opened file
          foreach ($order->get_items() as $item_id => $item) {

            // Get product handle
            $product = $item->get_product();

            // Get the product ID
            $product_id = $product->get_id();

            $buffer = $product_id . ';"purchase";' . $user_id . ';"'.$order->get_date_created()->format(DateTime::ATOM) .'"'. "\n";

            // Write to file
            fwrite($fp, $buffer);

          }
        }
      }
      fclose($fp);

  }

  function export_data() {
      $this->cron_items_dump();
      $this->cron_users_dump();
      $this->cron_interactions_dump();
  }

  function register_settings() {
      register_setting('visidea_plugin_options', 'visidea_plugin_options', 'visidea_plugin_options_validate');
      add_settings_section('api_settings', __( 'API Settings', 'visidea' ), 'visidea_plugin_section_text', 'visidea_plugin');
      add_settings_field('visidea_plugin_setting_public_token', __( 'Public token', 'visidea' ), 'visidea_plugin_setting_public_token', 'visidea_plugin', 'api_settings');
      add_settings_field('visidea_plugin_setting_private_token', __( 'Private token', 'visidea' ), 'visidea_plugin_setting_private_token', 'visidea_plugin', 'api_settings');
      add_settings_field('visidea_plugin_setting_database', __( 'Database', 'visidea' ), 'visidea_plugin_setting_database', 'visidea_plugin', 'api_settings');
      add_settings_section('api_labels', __( 'File URLs', 'visidea' ), 'visidea_plugin_section_labels', 'visidea_plugin');
  }


  function add_settings_page() {

      add_menu_page( "Visidea", "Visidea", "manage_options", "visidea", "visidea_render_plugin_index_page", plugins_url('images/visidea-icon.png', __FILE__) , 8);
      add_submenu_page("visidea", __( 'Settings', 'visidea' ), __( 'Settings', 'visidea' ), "manage_options", "visidea_general_setting","visidea_render_plugin_settings_page");

  }

  public function callback_new_item() {
    $this->cron_items_dump();
  }

  public function callback_new_user() {
    $this->cron_users_dump();
  }

  public function callback_new_interaction() {
    $this->cron_interactions_dump();
  }

}

function visidea_render_plugin_index_page() {
    ?>
    <img  src="<?php echo plugins_url('images/visidea-logo.png', __FILE__); ?>" style="width:350px">
    <hr>
    <h2><?php _e( 'Welcome to Visidea', 'visidea' ); ?></h2>

    <span><?php _e( 'Visidea is the visual search and visual recommendation plugin for Woocommerce.', 'visidea' ); ?></span><br><br>
    <div>
        <span><?php _e( 'This plugin need an account on Visidea.ai. You can subscribe ', 'visidea' ); ?><strong><a href="https://visidea.ai/pricing/" target="_blank"><?php _e( 'here', 'visidea' ); ?></a></strong></span><br><br>
        <span><?php _e( 'Discover more about our service on our ', 'visidea' ); ?><strong><a href="https://visidea.ai/" target="_blank"><?php _e( 'website', 'visidea' ); ?></a></strong></span><br><br>
        <span><?php _e( 'Go to Visidea dashboard to get the ', 'visidea' ); ?><strong><a href="https://app.visidea.ai/" target="_blank"><?php _e( 'API keys', 'visidea' ); ?></a></strong></span><br><br>
        <span><?php _e( 'Any problems? contact us: ', 'visidea' ); ?><strong><a href="mailto:support@visidea.ai"><?php _e( 'support@visidea.ai', 'visidea' ); ?></a></strong></span>
    </div>

    <?php
}

function visidea_render_plugin_settings_page() {
    ?>
    <img  src="<?php echo plugins_url('images/visidea-logo.png', __FILE__); ?>" style="width:350px">
    <hr>
    <h2><?php _e( 'Visidea configurations', 'visidea' ); ?></h2>
    <form action="options.php" method="post">
        <?php
        settings_fields('visidea_plugin_options');
        do_settings_sections('visidea_plugin'); ?>
        <input name="submit" class="button button-primary" type="submit" value="<?php _e( 'Save', 'visidea' ); ?>" />
    </form>
    <?php
}

function visidea_plugin_section_text() {
    echo '<p>'.__( 'Here you can set all the options for using our API:', 'visidea' ).'</p>';
}

function visidea_plugin_setting_public_token() {
    $options = get_option('visidea_plugin_options');
    $public_token = isset($options['public_token'])?$options['public_token']:'';
    echo "<input id='visidea_setting_public_token' name='visidea_plugin_options[public_token]' type='text' value='".esc_attr($public_token)."' />";
}

function visidea_plugin_setting_private_token() {
    $options = get_option('visidea_plugin_options');
    $private_token = isset($options['private_token'])?$options['private_token']:'';
    echo "<input id='visidea_setting_private_token' name='visidea_plugin_options[private_token]' type='text' value='".esc_attr($private_token)."' />";
}

function visidea_plugin_setting_database() {
    $options = get_option('visidea_plugin_options');
    $database = isset($options['database'])?$options['database']:'';
    echo "<input id='visidea_setting_database' name='visidea_plugin_options[database]' type='text' value='".esc_attr($database)."' />";
}

function visidea_plugin_section_labels() {
    $options = get_option('visidea_plugin_options');
    $private_token = isset($options['private_token'])?$options['private_token']:'';
    if (empty($private_token)) {
        echo '<p>'.__( 'You have to setup tokens to get file urls.', 'visidea' ).'</p>';
    } else {
        echo '<p>'.__( 'Items file:', 'visidea' ).'</p>';
        echo '<p><pre>'.plugins_url('items_'.esc_attr($private_token).'.csv', __FILE__).'</pre></p>';
        echo '<p>'.__( 'Users file:', 'visidea' ).'</p>';
        echo '<p><pre>'.plugins_url('users_'.esc_attr($private_token).'.csv', __FILE__).'</pre></p>';
        echo '<p>'.__( 'Interactions file:', 'visidea' ).'</p>';
        echo '<p><pre>'.plugins_url('interactions_'.esc_attr($private_token).'.csv', __FILE__).'</pre></p>';
    }
}

function visidea_plugin_options_validate($input) {
    // var_dump($input);exit;
    // $newinput['api_key'] = trim($input['api_key']);
    $newinput['public_token'] = trim($input['public_token']);
    $newinput['private_token'] = trim($input['private_token']);
    $newinput['database'] = trim($input['database']);
    // if (!preg_match('/^[a-z0-9]{32}$/i', $newinput['api_key'])) {
    //     $newinput['api_key'] = '';
    // }
    // var_dump($newinput);exit;
    return $newinput;
}
