<?php
/**
 * @wordpress-plugin
 * Plugin Name: Visidea
 * Plugin URI: https://visidea.ai
 * Description: Visidea is the visual search and visual recommendation plugin for WooCommerce. Visidea improves UX and increases the revenues of your website.
 * Version: 1.0.2
 * Author: Inferendo
 * Author URI: https://www.visidea.ai
 * License: GPLv3
 * License URI: https://www.gnu.org/licenses/gpl-3.0.html
 * Text Domain: visidea
 * @package Visidea
 */

// If this file is called directly, abort.
if (!defined( 'WPINC')) {
	die;
}

/**
 * Require action scheduler.
 */
require_once( plugin_dir_path( __FILE__ ) . '/libraries/action-scheduler/action-scheduler.php' );

/**
 * Current Visidea version.
 */
define('VISIDEA_VERSION', '1.0.2');

/**
 * Define a custom cronjob time interval to use later,
 * during activation.
 */
// function custom_cron_interval_filter($schedules) {
//
//   $schedules['half_hour'] = array (
//     'interval' => 1800,
//     'display' => esc_html__('Every Half Hour'),
//   );
//
//   return $schedules;
// }
// add_filter('cron_schedules', 'custom_cron_interval_filter', 10, 1);

/**
 * The code that runs during Visidea activation.
 * This action is documented in includes/class-visidea-activator.php
 */
function activate_visidea() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-visidea-activator.php';
    Visidea_Activator::activate();
}
register_activation_hook( __FILE__, 'activate_visidea' );

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-plugin-name-deactivator.php
 */
function deactivate_visidea() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-visidea-deactivator.php';
	Visidea_Deactivator::deactivate();
}
register_deactivation_hook( __FILE__, 'deactivate_visidea' );

/**
 * The core Visidea class that is used to define admin-specific hooks
 * and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-visidea.php';

/**
 * Begins execution of Visidea.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_visidea() {
	$plugin = new Visidea();
	$plugin->run();
}

run_visidea();

/**
 * Schedule an action with the hook 'eg_midnight_log' to run at midnight each day
 * so that our callback is run then.
 */
function visidea_schedule_export_data() {
	if ( false === as_has_scheduled_action( 'visidea_export_data_job' ) ) {
		as_schedule_recurring_action( strtotime( '+1 minutes' ), 3600, 'visidea_export_data_job' );
	}
}
add_action( 'init', 'visidea_schedule_export_data' );

/**
 * A callback to run when the 'eg_midnight_log' scheduled action is run.
 */
function visidea_export_data_job() {
	// error_log( 'Visidea export at ' . date( DATE_RFC2822 ) );
	$plugin_admin = new Visidea_Admin('visidea', VISIDEA_VERSION);
	$plugin_admin->export_data();
}
add_action( 'visidea_export_data_job', 'visidea_export_data_job' );
